# -*- coding: utf8 -*-
import traceback

import urllib3
try:
    from utils.util_normal import get_proxy_ip
except:
    pass
urllib3.disable_warnings()
import requests

reqs = requests.session()
reqs.keep_alive = False


class Downloader(object):
    """
    通用下载器
    """

    def __init__(self, task_name):
        self.task_name = task_name

    def get(self, request, timeout: int = 30, request_proxy=False):
        """获取页面Dom结果
        Args:
            request: SpiderRequest对象
        """
        try:
            req = request.urls[0]
            url = req["url"]
            method = req.get("method", "get").upper()             # 请求类型
            headers = request.headers                              # 请求头
            request_parms = req.get("request_parms", {})
            params = request_parms.get("params", None)            # 作为参数增加到URL中
            data = request_parms.get("data", None)                # POST时，data其格式必须为字符串
            json_ = request_parms.get("json", None)               # POST时，json数据，直接传dict
            verify = request_parms.get("verify", None)            # 验证
            proxies = request_parms.get("proxies", None)          # 代理
            if request_proxy:
                proxy_dict = get_proxy_ip()
                if proxy_dict:
                    proxies = {
                        "http": f"http://{proxy_dict['username']}:{proxy_dict['password']}@{proxy_dict['ip']}:{proxy_dict['port']}",
                        "https": f"http://{proxy_dict['username']}:{proxy_dict['password']}@{proxy_dict['ip']}:{proxy_dict['port']}"
                    }
                else:
                    print(f'{self.task_name} 获取代理ip失败')
            response = requests.request(method=method, url=url, headers=headers, params=params, data=data, json=json_,
                                    proxies=proxies, verify=verify, timeout=timeout)
            return {"success": True, "results": response}
        except requests.exceptions.ConnectTimeout:
            # 代理问题 连接超时
            print(traceback.format_exc())
        except Exception:
            print(traceback.format_exc())
            return {"success": False, "message": str(traceback.format_exc())}


class SpiderRequest(object):

    __slots__ = ['headers', 'config', 'urls']    # save memory

    def __init__(self, headers=dict(), config=dict(), urls=list()):
        self.headers = headers
        self.config = config
        self.urls = urls


# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/26
# description: 案例使用
import os
import sys

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)

from hgj_download_center.downloader.downloader import SpiderRequest
from hgj_download_center.downloader.new_basespider import NewBaseSpider

class  DemoTest(NewBaseSpider):
    def __init__(self):
        super(DemoTest, self).__init__(task_name="task", use_redis=False)

    def start_requests(self):
        url = "https://www.baidu.com"
        headers = {
                "Connection": "keep-alive",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.71 Safari/537.36",
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
                "Sec-Fetch-Mode": "navigate",
                "Sec-Fetch-Dest": "document",
            }
        urls = [{"url": url, "method": "GET", "retry_num": 1}]
        request = SpiderRequest(headers=headers,urls=urls)
        self.sending_queue.put(request)

    def deal_response_results_status(self, task_status, url, result, request):
        if task_status == 2:
            print(result["result"])


if __name__ == '__main__':
    a = DemoTest()
    a.run(1, 1, 1, 1)


# -*- coding:utf-8 -*-
"""
处理任务工具
"""
import datetime
import os
import sys
import json
import time


PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)
from utils.util_data_interaction import UtilData  # 报错：需自行配置对应接口的请求
from functools import reduce
from hgj_download_center.util.util_common import reset_redis_task_status


class DealTtool(object):
    def __init__(self, task_name, redis_serve, redis_all_prefix, redis_prefix):
        self.task_name = task_name
        self.util_data = UtilData()
        self.redis = redis_serve
        self.redis_all_prefix = redis_all_prefix
        self.redis_prefix = redis_prefix

    def put_dict_and_check(self, task_name_list: list):
        """ 获取新任务存入redis并去重 """
        new_redis_dict = dict()
        new_task_index_list = []  # 存新任务的索引值
        task_list = self.util_data.get_data(task_name_list)
        existing_tasks_index = self.get_existing_all_tasks()

        for task_dict in task_list:
            redis_key = self.redis_prefix + task_dict['boxNo']
            if self.redis.exists(redis_key) == 1:continue
            task_dict['task_status'] = 0
            new_redis_dict[redis_key] = json.dumps(task_dict)
            new_task_index_list.append(redis_key)

        # 批量放入redis
        if len(new_redis_dict.keys()) > 0:
            self.redis.mset(new_redis_dict)

        # 将单个任务索引存表
        if existing_tasks_index is None:
            existing_tasks_index = []
        existing_tasks_index.extend(new_task_index_list)
        func = lambda x, y: x if y in x else x + [y]
        existing_tasks_index = reduce(func, [[], ] + existing_tasks_index)
        self.redis.set(self.redis_all_prefix, json.dumps(existing_tasks_index))

    def get_existing_all_tasks(self, first_send=False) -> list:
        """ 获取redis中指定船司的所有索引"""
        if first_send:
            existing_tasks_index = self.get_existing_all_2()
            self.redis.set(self.redis_all_prefix, json.dumps(existing_tasks_index))
        else:
            existing_tasks_index = self.redis.get(self.redis_all_prefix)
            existing_tasks_index = [] if existing_tasks_index is None else json.loads(existing_tasks_index)
        return existing_tasks_index

    def get_existing_all_2(self, search_num=1000) ->list:
        """ 首次启动脚本需要 模板查询一次"""
        if search_num < 1000: search_num = 1000
        k = 0
        data_list = []
        while True:
            try:
                l = self.redis.scan(k, count=search_num)
                data_list.extend(i for i in l[1] if self.redis_prefix in i)
                k = l[0]
                if k == 0:
                    break
            except:
                break
        return data_list

    def get_redis_keys(self, first_send=False):
        """ 获取redis-keys"""
        existing_tasks_index = self.get_existing_all_tasks(first_send)
        return existing_tasks_index

    def get_sending_redis_tasks(self) -> list:
        """ 获取待发送的任务"""
        keys = self.get_redis_keys()
        redis_tasks = []
        for task_dict in self.redis.mget(keys):
            if type(task_dict) is None or type(task_dict) != str:
                continue
            task_dict = json.loads(task_dict)
            next_query_time = task_dict.get('next_query_time', '')
            if next_query_time:
                next_query_time = datetime.datetime.strptime(next_query_time.replace(' ', 'T'), '%Y-%m-%dT%H:%M:%S')
                if datetime.datetime.now() - datetime.timedelta(hours=2) > next_query_time and task_dict["task_status"] == 1:
                    task_dict["task_status"] = 0
                    self.redis.set(self.redis_prefix + task_dict['boxNo'], json.dumps(task_dict))
            redis_tasks.append(task_dict)
        return redis_tasks

    def update_redis_data(self, task_dict, item, task_name=""):
        """ 更新查询完的数据更新"""
        takeHeavyBoxTime = item.get("takeHeavyBoxTime", "")
        backEmptyBoxTime = item.get("backEmptyBoxTime", "")
        new_task_dict = dict(task_dict, **item)
        self.util_data.reply_res(new_task_dict)  # 先返回数据
        if task_name in ["ial", "kkc"] and not takeHeavyBoxTime:
            reset_redis_task_status(new_task_dict, self.redis, self.redis_prefix, )
        elif task_name == "" and not backEmptyBoxTime:
            reset_redis_task_status(new_task_dict, self.redis, self.redis_prefix, )
        else:
            self.send_res(new_task_dict)
            self.redis.delete(self.redis_prefix + task_dict['boxNo'])
            self.update_redis_index(task_dict)

    def update_redis_index(self, task_dict):
        """ 更新redis索引相关"""
        existing_tasks_index = self.get_existing_all_tasks()
        redis_key = self.redis_prefix + task_dict['boxNo']
        try:
            existing_tasks_index.remove(redis_key)
        except:
            pass
        self.redis.set(self.redis_all_prefix, json.dumps(existing_tasks_index))

    def send_res(self, callback_data):
        """发送已标记的回执"""
        self.util_data.destination_refuse(callback_data)

    def check_redis_data(self):
        """ 程序启动 重置所有任务状态"""
        keys = self.get_redis_keys(first_send=True)
        redis_tasks = [json.loads(i) for i in self.redis.mget(keys)]
        for task_dict in redis_tasks:
            redis_key = self.redis_prefix + task_dict['boxNo']
            if task_dict['task_status'] != 0:
                task_dict['task_status'] = 0
                self.redis.set(redis_key, json.dumps(task_dict))
                time.sleep(0.5)

    def update_error_data(self, task_dict):
        search_flag = True
        next_query_time = task_dict.get('next_query_time', '')
        if task_dict['task_status'] != 0 and (
                next_query_time == '' or datetime.datetime.strptime(task_dict['next_query_time'],"%Y-%m-%d %H:%M:%S") <= datetime.datetime.now()):
            task_dict['task_status'] = 0
            self.redis.set(self.redis_prefix + task_dict['boxNo'], json.dumps(task_dict))
        elif task_dict['task_status'] == 0 and (next_query_time == ''
            or datetime.datetime.strptime(task_dict['next_query_time'], "%Y-%m-%d %H:%M:%S") > datetime.datetime.now()):
            search_flag = False
        return search_flag



# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/25
import datetime
import os
import sys


PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(PROJECT_PATH)
from itsdangerous import json
from loguru import logger


def get_logger(task_name):
    logger.add(os.path.join(PROJECT_PATH, "logs", f"{task_name}.log"),
               level='DEBUG',
               format='{time:YYYY-MM-DD HH:mm:ss} - {level} - {line} : {message}',
               rotation="500 MB")
    return logger


def set_pid(pid_key):
    """设置pid"""
    if pid_key:
        pid_dict = get_pid_dict()
        pid_dict[pid_key.lower()] = int(os.getpid())

        with open(os.path.join(PROJECT_PATH, "docs", "pid.json"), "w") as f:
            json.dump(pid_dict, f)
        return True
    else:
        return False

def get_pid_dict():
    with open(os.path.join(PROJECT_PATH, "docs", "pid.json"), "r+") as f:
        pid_dict = json.load(f)
    return pid_dict


def reset_redis_task_status(task_dict, redis_ser, redis_prefix, update_time=True, minutes=120):
    """
    重置redis中任务状态和下次查询时间
    :param task_dict:       当前任务字典
    :param redis_ser:       redis连接对象
    :param redis_prefix:    redis-key 前缀
    :param update_time:     是否更新查询时间
    :param minutes:         下次任务查询间隔时间

    :return:
    """
    task_dict['task_status'] = 0
    if update_time:
        task_dict['next_query_time'] = (datetime.datetime.now() + datetime.timedelta(minutes=minutes)).strftime(
            "%Y-%m-%d %H:%M:%S")
    redis_ser.set(redis_prefix + task_dict['boxNo'], json.dumps(task_dict))





# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/26
# description: 
import os
import sys

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)



# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/18
# des: requests 工具类
import os
import sys
import time
import traceback
import requests

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)

from datetime import datetime
from queue import Queue
from queue import Empty
from threading import Thread
from hgj_download_center.downloader.downloader import Downloader
from hgj_download_center.store.py_store_redis_pool import StoreRedislPool
from hgj_download_center.util.util_common import get_logger, set_pid
from hgj_download_center.util.util_retry import retry
from hgj_download_center.util.util_useragent import UtilUseragent
from utils.config import REDIS_CONFIG  #  报错：需自行导入Redis配置
from hgj_download_center.util.util_deal_tool import DealTtool
from prettytable import PrettyTable


class NewBaseSpider(object):
    def __init__(self, task_name, use_redis=True, env=None, carrier_account=None):
        """
        基本的爬虫类,由队列控制；
        发送的对象统一封装成SpiderRequest对象
        启动时，通过start_requests方法构造一批待发送的任务对象送到待发送队列里面
        :param task_name:                   任务名称
        :param use_redis:                   是否使用redis
        :param env:                         开发环境
        :param carrier_account:             账号密码管理
        """
        self.task_name = task_name.lower()
        self.use_redis = use_redis
        if use_redis:
            self.redis = StoreRedislPool(REDIS_CONFIG).redis
            self.redis_prefix = f'track_{self.task_name}_'         # 单条任务键名
            self.redis_all_prefix = f'track_all_{self.task_name}'  # redis 索引名
            self.deal_tool = DealTtool(self.task_name, self.redis, self.redis_all_prefix, self.redis_prefix)
        self.logger = get_logger(self.task_name)
        self.pc_user_agents = UtilUseragent.get()
        self.mb_user_agents = UtilUseragent.get(type='MOBILE')
        self.sending_queue = Queue()
        self.sended_queue = Queue()
        self.response_queue = Queue()
        self.sended_queue_max = 5000     # sended_queue 最大值
        self.response_queue_max = 1500   # response_queue 最大值
        self.task_name = task_name
        self.request_proxy = False
        self.downloader = self.get_downloader()
        self.thread_wait = 0
        self.logger.info(f'程序启动：{env}')
        self.username = carrier_account.get(self.task_name.upper(), {}).get("username", "")
        self.password = carrier_account.get(self.task_name.upper(), {}).get("password", "")
        if sys.platform != "darwin":
            set_pid(pid_key=self.task_name)

    def get_downloader(self):
        """
        设置下载器类型，默认为Downloader
        Return:
            SpiderDownloader
        """
        return Downloader(self.task_name)

    def start_requests(self):
        """
        初始化待发送请求队列，由子类实现。拼装请求数据并送到sending_queue队列中
        todo: 2022-05: 当前只接收列表中存放单条任务的格式
        """
        raise NotImplementedError()

    def is_finish(self):
        """
        根据相关队列是否全都为空来判断任务处理结束
        """
        return self.sending_queue.empty() and self.sended_queue.empty() and self.response_queue.empty()

    def record_log(self):
        """
        记录抓取日志，用于调整各个线程参数设置
        """
        while True:
            table = PrettyTable(["x", "y"])
            table.add_row(["datetime", str(datetime.now())])
            table.add_row(["sending_queue", self.sending_queue.qsize()])
            table.add_row(["sended_queue", self.sended_queue.qsize()])
            table.add_row(["response_queue", self.response_queue.qsize()])
            table.reversesort = True
            print(table)
            del table
            # objgraph.show_most_common_types()
            time.sleep(30)

    def get_redis_tasks(self):
        """ 组装要发送的任务列表"""
        task_list = []
        redis_tasks = self.deal_tool.get_sending_redis_tasks()
        for task_dict in redis_tasks:
            next_query_time = task_dict.get('next_query_time', '')
            if task_dict['task_status'] == 0 and (
                    next_query_time == '' or datetime.strptime(task_dict['next_query_time'], "%Y-%m-%d %H:%M:%S") <= datetime.now()):
                task_list.append(task_dict)
        return task_list

    @retry(retry_times=-1, wait=5)
    def put_task_dict_into_redis(self):
        """ 从接口获取数据"""
        while True:
            self.deal_tool.put_dict_and_check([self.task_name])
            time.sleep(30 * 60)

    def deal_request_results(self, results, request):
        if results['success']:
            self.sended_queue.put(request)
        else:
            print('send parameters error urls: {}'.format(request.urls))

    @retry(retry_times=-1, wait=5)
    def send_requests(self, max_idle_time):
        results = dict()  # todo: 以后部署服务器后完善
        results['success'] = True
        start_time = time.time()
        while True:
            try:
                if self.sended_queue.qsize() < self.sended_queue_max and self.response_queue.qsize() < self.response_queue_max:
                    request = self.sending_queue.get(timeout=1)
                    self.deal_request_results(results, request)
            except Empty:
                pass
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(10)
            except Exception:
                print(traceback.format_exc())

    @retry(retry_times=-1, wait=5)
    def get_response(self, max_idle_time):
        """
        获取url爬取结果。将sended_queue队列中的SpiderRequest对象通过downloader到下载中心去获取抓取到的html
        """
        start_time = time.time()
        while True:
            try:
                if self.response_queue.qsize() < self.response_queue_max:
                    request = self.sended_queue.get(timeout=1)  # task_id,user_id,urls
                    results = self.downloader.get(request, request_proxy=self.request_proxy)       # request 对象
                    self.response_queue.put((request, results))  # 结果
                    start_time = time.time()
                    self.get_wait()     # wait confirm
                else:
                    time.sleep(5)
            except Empty:
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(5)
            except Exception:
                print(traceback.format_exc())

    @retry(retry_times=-1, wait=5)
    def deal_response(self, max_idle_time=-1):
        """
        从结果队列response_queue中取出结果进行处理
        """
        start_time = time.time()
        while True:
            try:
                request, results = self.response_queue.get(timeout=1)
                try:
                    self.deal_response_results(request, results)
                except Exception:
                    print(traceback.format_exc())
                start_time = time.time()
            except Empty:
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(10)
            except Exception:
                print(traceback.format_exc())

    def deal_response_results(self, request_, results):
        u = request_.urls[0]
        result = {"url": u["url"], "result": "", "header": "",
                  "redirect_url": "", "code": 0}

        if results['success']:
            response = results.get('results', '')
            code = response.status_code; result["code"]=code
            if code == 200:
                try:
                    if response.encoding == 'ISO-8859-1':
                        encodings = requests.utils.get_encodings_from_content(response.text)
                        if encodings:
                            encoding = encodings[0]
                        else:
                            encoding = response.apparent_encoding
                    else:
                        encoding = response.encoding
                except:
                    encoding = "utf-8"
                if result["url"] != response.url:
                    result["redirect_url"] = response.url
                encode_content = response.content.decode(encoding, 'replace').encode('utf-8', 'replace')
                if not isinstance(encode_content, str):
                    encode_content = encode_content.decode(encoding="utf-8", errors='ignore')  # bytes to str
                result["result"] = encode_content
                self.deal_response_results_status(2, u, result, request_)
            else:
                self.deal_response_results_status(2, u, result, request_)
        else:
            self.deal_response_results_status(3, u, result, request_)

    def deal_response_results_status(self, task_status, url, result, request):
        """
            处理 task_status 是2,3的任务  重试返回数组， 若重试需切换headers内容需自行定义
        :param task_status:
        :param url:
        :param result:
        :param request:
        :return:
        """
        raise NotImplementedError()

    def get_wait(self):
        """
        获取结果等待, 控制发往处理队列的速率
        """
        if self.thread_wait != 0:
            time.sleep(self.thread_wait)
        else:
            if self.response_queue.qsize() > 4000:
                time.sleep(5)
            elif self.sended_queue.qsize() < 2000:
                time.sleep(1)
            else:
                time.sleep(0.1)

    def run(self, put_num=0, get_num=1, send_num=0, deal_num=0, send_idle_time=-1,get_idle_time=-1,deal_idle_time=-1, record_log=False, spider_count=0):
        """
        爬虫启动入口
        Args:
        """
        if self.use_redis:
            self.deal_tool.check_redis_data()
        thread_start = Thread(target=self.start_requests)  # 发送请求
        thread_start.start()
        print("put_num: {}; send_num:{}; deal_num: {};  spider_count: {}".format(put_num, send_num, deal_num, spider_count))
        threads = list()

        put_num = 0 if not self.use_redis else put_num  # 不使用redis时，不会从接口中获取数据
        for i in range(0, put_num):
            threads.append(Thread(target=self.put_task_dict_into_redis, args=()))
        for i in range(0, get_num):
            threads.append(Thread(target=self.get_response, args=(get_idle_time,)))
        for i in range(0, send_num):
            threads.append(Thread(target=self.send_requests, args=(send_idle_time,)))
        for i in range(0, deal_num):
            threads.append(Thread(target=self.deal_response, args=(deal_idle_time,)))
        if record_log:
            thread = Thread(target=self.record_log)
            thread.setDaemon(True)
            threads.append(thread)
        for thread in threads:
            thread.start()


# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/18
# des: selenium 工具类

import os
import sys
import datetime
import zipfile
import requests
import time
from pyvirtualdisplay import Display
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from datetime import datetime

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)

from utils.config import PROXY_IP_URL
try:
    from utils.util_normal import get_proxy_ip  # 需替换自己的代理池
except:
    pass
from hgj_download_center.downloader.new_basespider import NewBaseSpider


class SeleniumSpiderBase(NewBaseSpider):
    def __init__(self, task_name, use_redis=True):
        super(SeleniumSpiderBase, self).__init__(task_name=task_name, use_redis=use_redis)
        self.task_name = task_name
        self.use_redis = use_redis
        # 是否使用 无头模式 无痕模式，默认为True
        self.is_headless, self.is_incognito = False, False
        # 是否加载图片
        self.load_picture = True
        self.use_userdata = False
        # 代理
        self.use_proxy = False
        self.ip_prefix = "ippool_"

    def create_browser(self):
        """
        初始化一个chrome
        """
        # desired_capabilities = DesiredCapabilities.CHROME  # 修改页面加载策略
        # desired_capabilities["pageLoadStrategy"] = "none"  # 注释这两行会导致最后输出结果的延迟，即等待页面加载完成再输出

        chrome_options = webdriver.ChromeOptions()

        # 设置中文
        chrome_options.add_argument('lang=zh_CN.UTF-8')

        # root
        chrome_options.add_argument('--no-sandbox')
        # 谷歌文档提到需要加上这个属性来规避bug
        chrome_options.add_argument('--disable-gpu')
        # 隐藏滚动条, 应对一些特殊页面
        chrome_options.add_argument('--hide-scrollbars')
        chrome_options.add_argument("disable-infobars")
        chrome_options.add_argument('--disable-dev-shm-usage')
        # 忽略验证：您的连接不是私密连接
        chrome_options.add_argument('--ignore-certificate-errors')

        # 取消chrome受自动控制提示
        chrome_options.add_experimental_option("excludeSwitches", ['enable-automation'])
        # 扩展程序
        # chrome_options.add_argument("--disable-extensions")
        # chrome_options.add_experimental_option('useAutomationExtension', False)

        if self.load_picture is False:
            # 不加载图片, 提升速度
            chrome_options.add_argument('blink-settings=imagesEnabled=false')

        # 关掉密码弹窗
        prefs = {"": ""}
        prefs["credentials_enable_service"] = False
        prefs["profile.password_manager_enabled"] = False
        chrome_options.add_experimental_option("prefs", prefs)

        if self.use_proxy is True:
            # 无头模式，无痕模式 不支持使用扩展
            self.is_headless = False
            self.is_incognito = False
            self.use_userdata = False
            while True:
                try:
                    proxy_dict = get_proxy_ip()
                    if proxy_dict.get('info', '') == '暂时没有代理ip':
                        self.logger.info('暂时没有代理ip')
                        time.sleep(10)
                        continue
                    else:
                        break
                except:
                    time.sleep(30)

            PLUGIN_PATH = self.__create_proxyauth_extension(proxy_host=proxy_dict['ip'], proxy_port=proxy_dict['port'], proxy_username=proxy_dict['username'], proxy_password=proxy_dict['password'])
            chrome_options.add_extension(PLUGIN_PATH)
            try:
                requests.delete(PROXY_IP_URL + "?ipkey={}".format(self.ip_prefix + proxy_dict['ip']), timeout=10)
            except:
                pass

        if self.use_userdata:
            # chrome_options.add_argument(f"--user-data-dir={os.path.join(PROJECT_PATH, 'userdata')}")
            pass

        if self.is_incognito:
            # 无痕
            chrome_options.add_argument('--incognito')

        if self.is_headless:
            chrome_options.add_argument('--headless')

        # 开启服务，通过服务，控制浏览器关闭
        if sys.platform == 'win32':
            executable_path = os.path.join(PROJECT_PATH, 'util', 'chromedriver')
        else:
            executable_path = '/usr/local/bin/chromedriver'
        try:
            selenium_service = Service(executable_path=executable_path)
            selenium_service.command_line_args()
            selenium_service.start()
            browser = webdriver.Chrome(executable_path=executable_path, options=chrome_options)
            browser.set_page_load_timeout(60)
        except:
            display = Display(visible=False, size=(1792, 1120))
            display.start()
            selenium_service = Service(executable_path=executable_path)
            selenium_service.command_line_args()
            selenium_service.start()
            browser = webdriver.Chrome(executable_path=executable_path, options=chrome_options)
            browser.set_page_load_timeout(60)
        try:
            stealth_path = os.path.join(PROJECT_PATH, 'util', 'stealth.min.js')
            with open(stealth_path) as f:
                js = f.read()
            browser.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
                "source": js
            })
        except:
            pass
        # try:
        #     tk = tkinter.Tk()
        #     width = tk.winfo_screenwidth()
        #     height = tk.winfo_screenheight()
        #     tk.quit()
        # except:
        width = 1792
        height = 1120
        browser.set_window_size(width, height)
        browser.maximize_window()

        browser.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
            "source": """
            Object.defineProperty(navigator, 'webdriver', {
              get: () => undefined
            })
          """
        })
        return selenium_service, browser

    def click_and_send_keys(self, browser, by, rule, msg):
        """
        点击并发送
        每次都获取是因为有时旧的元素定位不到会报错，所以每次都重新获取元素并完成相应操作
        :param browser: 浏览器对象
        :param by: 通过什么方式定位，如： id，xpath，css_selector
        :param rule: 定位的规则
        :param msg: 需要输入的值
        :return:
        """
        by = by.lower()
        self.__get_element(browser, by, rule).click()
        time.sleep(1)
        self.__get_element(browser, by, rule).clear()
        self.__get_element(browser, by, rule).click()
        time.sleep(1)
        self.__get_element(browser, by, rule).send_keys(msg)

    def clear_and_send(self, browser, by, rule, msg):
        """
        清空并发送
        每次都获取是因为有时旧的元素定位不到会报错，所以每次都重新获取元素并完成相应操作
        :param browser: 浏览器对象
        :param by: 通过什么方式定位，如： id，xpath，css_selector
        :param rule: 定位的规则
        :param msg: 需要输入的值
        :return:
        """
        by = by.lower()
        self.__get_element(browser, by, rule).clear()
        time.sleep(0.3)
        self.__get_element(browser, by, rule).send_keys(msg)
        time.sleep(0.3)

    @staticmethod
    def close_browser(selenium_service, browser):
        """
        关闭浏览器
        """
        try:
            browser.quit()
            time.sleep(1)
            selenium_service.stop()
            time.sleep(2)
        except:
            pass

    def init_new_browser(self, selenium_service, browser, next_init_time):
        """
        :param selenium_service: 服务
        :param browser: 浏览器
        :param next_init_time: 下次初始化时间
        :return:
        """
        if datetime.datetime.now() >= next_init_time:
            browser.quit()
            time.sleep(1)
            selenium_service.stop()
            time.sleep(2)

            selenium_service, browser = self.create_browser()
            next_init_time = datetime.datetime.now() + datetime.timedelta(hours=1)
        return selenium_service, browser, next_init_time

    def scroll_to_ele(self, browser, by, rule):
        """滚动到某个元素的位置"""
        ele = self.__get_element(browser, by, rule)
        js4 = "arguments[0].scrollIntoView();"
        browser.execute_script(js4, ele)
        return True

    def click_by_js(self, browser, by, rule):
        """通过js点击"""
        by = by.lower()
        ele = self.__get_element(browser, by, rule)
        browser.execute_script("arguments[0].click();", ele)
        time.sleep(0.5)

    def __create_proxyauth_extension(self, proxy_host, proxy_port, proxy_username, proxy_password, scheme='http'):
        """带密码验证的扩展"""
        plugin_path = os.path.join(PROJECT_PATH, "util", "proxy_auth_plugin.zip")

        manifest_json = """
                {
            "version": "1.0.0",
            "manifest_version": 2,
            "name": "Chrome Proxy",
            "permissions": [
                "proxy",
                "tabs",
                "unlimitedStorage",
                "storage",
                "<all_urls>",
                "webRequest",
                "webRequestBlocking"
            ],
            "background": {
                "scripts": ["background.js"]
            },
            "minimum_chrome_version":"22.0.0"
        }
                """
        background_js = """
            var config = {
            mode: "fixed_servers",
            rules: {
              singleProxy: {
                scheme: "http",
                host: "proxy_host",
                port: parseInt(proxy_port)
              },
              bypassList: ["foobar.com"]
            }
          };

        chrome.proxy.settings.set({value: config, scope: "regular"}, function() {});

        function callbackFn(details) {
            return {
                authCredentials: {
                    username: "proxy_username",
                    password: "proxy_password"
                }
            };
        }

        chrome.webRequest.onAuthRequired.addListener(
                callbackFn,
                {urls: ["<all_urls>"]},
                ['blocking']
        );
            """.replace('proxy_host', proxy_host).replace('proxy_port', str(proxy_port)) \
            .replace('proxy_username', proxy_username).replace('proxy_password', proxy_password)

        with zipfile.ZipFile(plugin_path, 'w') as zp:
            zp.writestr("manifest.json", manifest_json)
            zp.writestr("background.js", background_js)

        return plugin_path

    def __get_element(self, browser, by, value):
        """通过不同的方式查找界面元素"""
        element = None
        by = by.lower()
        if (by == "id"):
            element = browser.find_element_by_id(value)
        elif (by == "name"):
            element = browser.find_element_by_name(value)
        elif (by == "xpath"):
            element = browser.find_element_by_xpath(value)
        elif (by == "classname"):
            element = browser.find_element_by_class_name(value)
        elif (by == "css"):
            element = browser.find_element_by_css_selector(value)
        elif (by == "link_text"):
            element = browser.find_element_by_link_text(value)
        else:
            print("无对应方法，请检查")
        return element

    def send_msg_to_wechat(self, msg: str):
        """目的港预警机器人"""
        bot_url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=18688e74-3491-4ed6-8a7f-cc60721376c9'
        json_dict = {
            "msgtype": "text",
            "text": {
                "content": msg
            }
        }
        for i in range(3):
            res = requests.post(bot_url, json=json_dict)
            if res.status_code == 200:
                return True
        return False




# -*- coding:utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(name='hgj_spider_download',  # 包名
      version='1.0.3',  # 版本号
      description='hgj-下载器',
      long_description='##版本说明：\n - 1.0.3: 测试版 ,暂且仅支持发送页面请求，不支持sele',
      author='',
      author_email='',
      url='',
      license='',
      install_requires=[
          'requests==2.25.1',
          'redis==3.5.3',
          'gevent==21.1.2'

      ],
      classifiers=[
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Utilities'
      ],
      keywords='',
      packages=find_packages('src'),  # 必填，就是包的代码主目录
      package_dir={'': 'src'},  # 必填
      include_package_data=True,
      )
# !/usr/bin/env python




# -*- coding: utf-8 -*-
"""
重试机制，此装饰器只适用于装饰类中的方法，并且，调用的也是类中的方法，关键处在于：是否接收并传递self
"""
import os
import sys
import time
from inspect import isfunction
import traceback

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)
from utils.config import SERVER_ENV
from utils.util_normal import send_track_remind_wechat

def before_sleep(*args, **kwargs):
    pass


def after_sleep(*args, **kwargs):
    pass


def retry_error_callback(*args, **kwargs):
    pass


def retry(retry_times=5, current_times=0, wait=1, before_sleep=before_sleep, after_sleep=after_sleep, retry_error_callback=retry_error_callback, remark=""):
    """
    装饰器实现重试机制
    :param retry_times: 重试次数：默认5次，-1代表一直重试
    :param current_times: 当前重试的次数
    :param wait_fixed: 每次重试的时间间隔
    :param do_something_during_retry: 重试间隔需要执行的函数
    :param retry_error_callback: 重试结束仍然失败时的回调函数
    :return:
    """

    def catch_exception(func):
        def wrapper(self, *args, **kwargs):
            nonlocal retry_times, current_times
            try:
                # 调用func
                return func(self, *args, **kwargs)
            except Exception as e:
                send_track_remind_wechat(f"{SERVER_ENV}--" + str(traceback.format_exc()))
                current_times += 1
                # 睡眠前调用
                if (isfunction(before_sleep)):
                    before_sleep(self, current_times=current_times, remark=remark)

                # 重试间隔
                time.sleep(wait)

                # 睡眠后调用
                if (isfunction(after_sleep)):
                    after_sleep(self, current_times=current_times, remark=remark)

                if retry_times == -1:
                    return wrapper(self, *args, **kwargs)
                elif current_times > retry_times:
                    if (isfunction(retry_error_callback)):
                        # 重试结束，执行回调函数
                        return retry_error_callback(self, retry_times=retry_times, remark=remark)
                    else:
                        # 没有指定回调函数时，弹出错误
                        raise e
                else:
                    current_times += 1
                    return wrapper(self, *args, **kwargs)

        # 设置current_times参数值
        def set_current_times(value):
            nonlocal current_times
            current_times = value

        wrapper.set_current_times = set_current_times
        return wrapper

    return catch_exception



# -*- coding:utf-8 -*-
# author:jackwu 
# time:2022/5/18
import redis

class StoreRedislPool(object):
    """
    redis 读写相关操作
    Args：redis配置
    type：dict
    """
    def __init__(self, redis_config):
        conn_pool = redis.ConnectionPool(**redis_config, decode_responses=True)
        self.redis = redis.StrictRedis(connection_pool=conn_pool)
